/**************************************************************************************************
Copyright (c) 2005-2006 Darrell Tam
email: darrell.barrell@gmail.com


History
	Date       Version    Programmer         Comments
	16/2/03    1.0        Darrell Tam		Created


This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

***************************************************************************************************/
#include <dt/misc_stuff.h>
#include "GradColMap.h"

//------------------------------------------------------------------------------------------
void GradColMap::generate(
	int num_cols,		// number of colour entries
	float col_vec[][4],
	unsigned col_vec_n			// number of elements in col_vec
)
{
	// set array size to request number of colour entries
	resize(num_cols);

	// index into "col_vec"
	unsigned col_vec_i = 0;

	// colours being interpolated to & from
	float a[3], b[3];

	// interpolation fraction
	float frac, frac_step;

	// current index range into output data
	unsigned end_i = 0, beg_i;

	// get the RGB out of row-0 in "col_vec"
	memcpy(b, &col_vec[col_vec_i][1], sizeof(b));

	// generate the gradient using specified anchor points
	for(unsigned i = 0; i < size(); i++) {
		// chk whether we need to get the next element out of "col_vec"
		if(i >= end_i) {
			beg_i = end_i;
			memcpy(a, b, sizeof(a));
			if(col_vec_i < col_vec_n-1) col_vec_i++;

			end_i = (unsigned)(col_vec[col_vec_i][0]*(float)size());
			if(end_i < beg_i+1) end_i = beg_i+1;

			memcpy(b, &col_vec[col_vec_i][1], sizeof(b));
			frac_step = 1.0f/(float)(end_i-beg_i);
			frac = 0.0f;
		}

		union {
			unsigned char rgb[4];	// r,g,b,unused
			unsigned long col;		// ulong version of colour
		};

		for(int k = 0; k < 3; k++)
			rgb[2-k/*swap r & b for windows*/] = limit_range(
				(unsigned)(lin_interp(frac, a[k], b[k])*255.0f),
				0U, 255U
			);
		operator[](i) = col;

		frac += frac_step;
	}
}
