#ifndef _PIXEL_TO_FREQ_FRAC_H_
#define _PIXEL_TO_FREQ_FRAC_H_
/******************************************************************************
PixelFreqMap

  Given a pixel position (in the spectrogram or frequency control) return
  frequency fraction (0..1)

  Uses buzz param "freq_value" function to calculate frequency values

  Result is linear to exponential mapping


History
	Date       Version    Programmer         Comments
	14/01/03    -         Darrell Tam		Created (Buzz version)
	01/12/05    1.0       Darrell Tam		Ported to VST version

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

******************************************************************************/

#include <valarray>
#include <dt/cplx.h>
#include "rfftw_float.h"

//------------------------------------------------------------------------------------------
class PixelFreqMap
// generate:
//    pixel to frequency mappings that matches FreqParam
//    a series of pcplxf_range's to generate a spectrogram from fftw frequency data
{
public:
	PixelFreqMap();
	virtual ~PixelFreqMap();

	// generate mapings
	void build(
		int n_pixels,				// number of pixels wide
		float sample_freq,			// Hz
		float* fft_dat				// fft data as produced by fftw
	);	

	// get number of pixels wide
	int numPixels(void) const { return _pix_to_freq.size(); }

	// map a pixel to a frequency, pixel must be in range 0..(n_pixels-1)
	float/*hz*/pixToFreq(int pixel) const
		{ return _pix_to_freq[pixel]; }

	// return cplx ranges to map to pixels (note: don't use const version of [] since "&"
	// will return the address of result on the stack)
	pcplxf* pixToBinRng(int fft_idx)
		{ return &(_pix_to_binrng[fft_idx][0]); }

	// number of pixels per octave
	float pixPerOctave() { return _pix_per_octave; }

//protected:
	// number of pixels per octave
	float _pix_per_octave;

	// pixel to freq (hz) (may go beyond 1/2 sample rate)
	std::valarray<float> _pix_to_freq;

	// map pixel to bin for each fft length
	// frng[blk len][pixel]
	struct VecBinRng : public std::valarray<pcplxf> {}; // do this to stop MSVC debug complaining
	VecBinRng _pix_to_binrng[N_FFT_BLK_SZ];
};

#endif
