#ifndef _DT_SPECTROGRAM_H_
#define _DT_SPECTROGRAM_H_
/**************************************************************************************************
Copyright (c) 2005-2006 Darrell Tam
email: darrell.barrell@gmail.com

Pls see Spectrogram.cpp

History
	Date       Version    Programmer         Comments
	16/2/03    -        Darrell Tam		Buzz version
    01/12/05   1.0        DT                 ported to VST

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

***************************************************************************************************/

#include <dt/misc_stuff.h>
#include <dt/VstGuiSupport.h>

#include "PixelFreqMap.h"
#include "GradColMap.h"


//------------------------------------------------------------------------------------------
class Spectrogram : public /*VSTGUI*/CView
{
public: // initialization routines that MUST be called before use

	Spectrogram();
	virtual ~Spectrogram();

	// set the colour mapping and 
	void set(GradColMap& /*in*/ col_map);

	// set the frequency-pixel mapping & initialize (must call before use)
	void init(CRect size, PixelFreqMap& /*in*/ freq_map);

	// call when main effect suspends
	void suspend();

	// supply new FFT data to display
	void newData(
		long samp_pos,		// sample position
		long time_fft_n,	// number of samples in time domain (may be less samples than fft freq data)
		int fft_idx,		// fft index as per freq_map
		float pwr_scale		// scaling to apply to data
	);

	// set vertical line using pixel position (as an offset into the client rect)
	void setVLinePix(int x) { _vline_x[1] = limit_range(x, 0, _freq_map->numPixels()-1); }

	// set vertical line using a fraction
	void setVLineFrac(float/*0..1*/v) { setVLinePix((int)(v*(float)_freq_map->numPixels())); }

	// clear the vertical line
	void clrVLine() { _vline_x[1] = -1; }

	// height of tick at bottom in pixels
	int tick_height;

	// frame & tick colour
	CColor frame_col;

	// minimum number of samples per line of display (to limit scrolling speed)
	int min_samps_per_line;

	// client area (display area within borders & octave ticks)
	CRect client_rect;

	// text rectangle to display mouse position info
	CRect txt_rect;

	// message rectange to display "disabled" msg when paused
	CRect msg_rect;

	// whether or not we're paused
	bool paused;

public:	// override CView methods
	virtual void draw(CDrawContext *context);
	virtual void update(CDrawContext *context);
	virtual void mouse(CDrawContext *context, CPoint &where);
	virtual bool removed(CView* parent);

protected: // internals
	bool /*msg exists*/ writeMsg(CDrawContext *context);

	unsigned long mapCol(float in);

	void getMaxVals(
		bool empty_line,	// whether this is a new line (pass as constant to help optimizer)
		int fft_idx,		// fft index
		float pwr_scale
	);

	void draw (
		CDrawContext *context,	//
		CRect clip_rect			// limit drawing to this clipping rectangle
	);

	bool drawVLine(CDrawContext *context);

protected:
	// map to use for converting FFT bin pwr into a colour
	Ptr<GradColMap> _col_map;

	// colour scaling of log(pwr(fft data))
	float _col_log_offs, _col_log_scale;

	// map to turn pixel position into frequency and bin position
	Ptr<PixelFreqMap> _freq_map;

	// scaling to use on FFT bin pwr when mapping to a colour
	float _pwr_min, _pwr_max;

	// image data height in lines (may be different from size.height())
	int _image_height;

	// 2D image data array, 24 bit colour (last byte of every long is wasted)
	// lines are arranged in a circular buffer
	std::valarray<unsigned long> _image_data;

	// current line position in image_data & _samp_pos
	int _image_i;

	// we can display up to this line
	int _disp_i[2];			// [0]=currently displayed, [1]=can display to

	// sample position of each line in the image_data
	std::valarray<long> _line_samp_pos;

	// current min & max of each pixel
	std::valarray<float> _curr_max;

	// sample position of previously displayed line
	long _curr_line_samp_pos;

	// true if a line has just been output
	bool _curr_line_is_empty;

	// vertical line under mouse
	int _vline_x[2];		// [0]=current, [1]=new value

	// 
	MouseUpdate _mouse;

	//
	bool _ok;

	//
	bool _focus;
};

#endif
